#!/usr/bin/perl

# Copyright 2011 Ryan Babchishin
my $version = '2.08b';

=begin GPL
    This file is part of Moscrack.

    Moscrack is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    Moscrack is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moscrack.  If not, see <http://www.gnu.org/licenses/>.
=end GPL
=cut

my $refresh = 2; # Refresh time in seconds


use POSIX qw(strftime);
use Storable qw(lock_retrieve);
use strict;
use DateTime;
use Math::Round;
use Term::ANSIColor;
use DateTime::Format::Duration;
use File::Basename;
use CGI;
use LWP::Simple;
use JSON;
use Config::Std;



my $cgi = CGI->new();
print $cgi->header();
print "<font face=arial>\n";



## Read config file
our $configFile = '/etc/moscrack/moscrack.conf';
our $cfg = loadConfig();

my @nodeState = ('OFFLINE','ONLINE');
my $status;
my $activeNodes;
my $nodeStatus;
my $nodes;
my $key;
my $timer->{rate} = 0;
my $timerInterval = 600;


my $dataSource = 'API';
 unless($cfg->{moscapid}->{enabled}) {
   unless(-f 'status.dat') {
 	print "Can't find status.dat\n";
 	print "Either you are not in the Moscrack working directory or Moscrack hasn't started yet.\n";
 	exit 1;
   } else {
 	$dataSource = 'status.dat'
   } 
}


# Decide what page to load
if(defined($cgi->param('launchMoscrack'))) {
	launchMoscrack();
} else {
	mainPage();
}

exit;


sub mainPage() {
	print $cgi->start_form();
	print $cgi->submit(-name=>'launchMoscrack', -value=>'Launch');
	print " Auto-Refresh: ";
	print $cgi->submit(-name=>'refresh', -value=>'Off');
	print $cgi->submit(-name=>'refresh', -value=>'On');
	print $cgi->end_form();

	
        # Check if we are using the API
        if($cfg->{moscapid}->{enabled}) {
          my $json = JSON->new;
          # Keep going, even on failed connection
          my $statusJSON;
          unless($statusJSON = get($cfg->{moscapid}->{url}.'?action=get&value=status')) {
                warn "Error connecting to moscapid at $cfg->{moscapid}->{url}\n";
                sleep 1;
                next;
          }
          $status = $json->decode($statusJSON);
        } else {
                $status = lock_retrieve('status.dat');
        }

	$activeNodes = $status->{activeNodes};
	$nodeStatus = $status->{nodeStatus};
	$nodes = $status->{nodes};

	my $percentDone;
	my $complete;
	unless($status->{wordListSize} <= 0) {
		$percentDone = round(($status->{completeWords} / $status->{wordListSize}) * 100).'%';
		$complete = $status->{completeWords}.'/'.$status->{wordListSize};
	} else {
		$complete = $status->{completeWords};
		$percentDone = '';
	}

	if($status->{key}) {
		$key = "<font color=red><b>Key Found<br><font color=blue>".$status->{key};
	}

	my $pid;
	my $isRunning = 0;
	if(-d "/proc/$status->{pid}") {
		# Make sure this pid is "moscrack"
		open CMDLINE, "</proc/".$status->{pid}."/cmdline";
		my $cmdline = <CMDLINE>;
		close CMDLINE;

		my (undef,$cmdName) = split("\0",$cmdline);
		if ($cmdName =~ /moscrack/) {
			$isRunning = 1;
		}
	} 


	if ($isRunning == 1) {
		$pid = $status->{pid};
		print "<form method=get action=$cfg->{moscapid}->{url} target=_blank>";
                print "<input type=hidden name=action value=exec>";
                print "<input type=hidden name=value value=shutdown>";
                print "<input type=submit name=shutdown value=Shutdown>";
                print "</form>";
	} else {
		$pid = '<font color=red><b> Not running';
	}

	# View log button
	print "<form action=/moscrack/js-logtail?noreverse method=post target=_blank>";
	print "<input type=submit name=log value=Log>";
	print "</form>";


	my $b = ' bgcolor=black ';
	my $f = '<b><font color=white>';

	print "<table border=0 cellpadding=8 cellspacing=0>\n";
	print "<tr><td $b colspan=7 align=center><font color=white><b>Moscrack Status Monitor</b></td>\n";
	print "<tr><td $b>".$f."PID</td><td>$pid</td><td $b>".$f."ESSID</td><td>$status->{essid}</td>
		<td rowspan=5 colspan=2 align=center>$key</td></tr>\n";
	print "<tr><td $b>".$f."Word List</td><td>".basename($status->{wordlist})."</td><td $b>".$f."Complete</td><td>$complete</td></tr>\n";
	print "<tr><td $b>".$f."Processes</td><td>$status->{procCount}</td><td $b>".$f."Progress</td><td>$percentDone</td></tr>\n";
	print "<tr><td $b>".$f."Average Rate</td><td>$status->{wordPerSec}</td><td $b></td><td></td></tr>\n";
	print "</table>";

	my $w = "<font color=white>";
	print $cgi->start_table({-border=>0,-cellspacing=>0,-cellpadding=>8});
	print $cgi->Tr(
                  [
                     $cgi->th({-align=>'left',-bgcolor=>'black'},
			[$w.'Node', $w.'Type',$w.'Mult',$w.'LastRes',$w.'Status',$w.'Fail',$w.'Error',$w.'Throt',
				$w.'PID',$w.'Time',$w.'Chunk'])
                  ]
                  );

	foreach my $node (keys %{$nodes}) {

		my ($pid, $chunk, $startTime) = doingWhat($node);

		my $pidSeconds = DateTime->now->epoch - $startTime if $startTime > 0;
		my $pidTime = strftime("\%H:\%M:\%S", gmtime($pidSeconds));

		my $throttleTime;
		if (defined($nodeStatus->{$node}->{throttle})) {
			$throttleTime = $nodeStatus->{$node}->{throttle} - DateTime->now->epoch;
			if ($throttleTime <= 0) {
				$throttleTime = undef; 
			} else {
				$throttleTime = strftime("%M:\%S", gmtime($throttleTime));	
			}
		}


		my $nodeName = $node;
		$nodeName =~ s/^(.{16}).+/$1/;	

		my $plugin;
		$plugin = $nodes->{$node}->{plugin}.'/' if $nodes->{$node}->{plugin};

		print $cgi->Tr(
                  [
                     $cgi->th({-align=>'left',-bgcolor=>'lightblue'},
			[
				$nodeName,
				$plugin.$nodes->{$node}->{type},
				$nodes->{$node}->{multiplier},
				$nodeStatus->{$node}->{lastResult},
				$nodeState[$nodeStatus->{$node}->{status}],
				$nodeStatus->{$node}->{fail},
				$nodeStatus->{$node}->{error},
				$throttleTime,
				$pid,
				$pidTime,
				$chunk
			])
		]);
	}


	print $cgi->end_table();
	print "<meta http-equiv=Refresh content=3>" unless $cgi->param('refresh') eq 'Off';
}

sub doingWhat {

	my $node = shift;

	foreach my $pid (keys %{$activeNodes}) {
		if ($node eq $activeNodes->{$pid}->{node}) {
			return $pid, $activeNodes->{$pid}->{chunk}, $activeNodes->{$pid}->{startTime};
		}			
	}

}

# Load configuration file
sub loadConfig {
        my $cfg;
        read_config($configFile, $cfg);
        return $cfg;
}

sub launchMoscrack {
        my $json = JSON->new;
        # Keep going, even on failed connection
        my $wordFilesJSON;
        unless($wordFilesJSON = get($cfg->{moscapid}->{url}.'?action=get&value=word_files')) {
              warn "Error connecting to moscapid at $cfg->{moscapid}->{url}\n";
              sleep 1;
              next;
        }
        my $wordFiles = $json->decode($wordFilesJSON);

        print $cgi->start_form();
	print "<select>\n";
	foreach my $fileName (@{$wordFiles}) {
		print "<option value=\"$fileName\">$fileName</option>\n";
	}
	print "</select>\n";

        print $cgi->submit(-name=>'start', -value=>'Start');

	print $cgi->end_form();

}
